function [aligned,offsets] = mtbalign(images,refIndex,crop)
%MTBALIGN Align a set of RGB images using the MTB algorithm.
%   aligned = ALIGN(images) returns a cell array of images aligned to each
%   other using the MTB algorithm. The input images should be provided as a
%   cell array of RGB images. The algorithm is only applied to the region
%   specified by crop ([top left; bottom right]). The aligned images will
%   all have the same dimensions which may be different than the dimensions
%   of the input images. The function also returns the a cell array of the
%   [x y] offsets between the images specified relative to images{refIndex}.

P = length(images);

% Convert images to grayscale
I = cell(P,1);
top = crop(1,1);  left = crop(1,2);
bottom = crop(2,1);  right = crop(2,2);
for i = 1:P
    I{i} = rgb2gray(images{i}(top:bottom,left:right,:));
end

% Create pyramid of median threshold bitmaps
levels = 3;
B = cell(P,levels);
N = cell(P,levels);
for level = 1:levels
    scale = 2^-(levels-level);
    for i = 1:P
        L = imresize(I{i},scale,'box');
        med = median(L(:));
        % Median threshold bitmap
        B{i,level} = L > med;
        % Threshold noise exclusion bitmap
        N{i,level} = ((L < (med - 5)) | (L > (med + 5)));
    end
end

% Determine offsets
offsets = cell(P,1);
offsets{refIndex} = [0 0];
others = [1:refIndex-1 refIndex+1:P];
for i = others
    px = 0;
    py = 0;
    for level = 1:levels
        [h,w] = size(B{i,level});
        px = px * 2;  py = py * 2;
        minerr = Inf;
        minx = 0;  miny = 0;
        for ox = -1:1
            for oy = -1:1
                x = px + ox;  y = py + oy;
                % Get overlapping regions of the shifted bitmaps
                ix = max(x+1,1):min(x+w,w);
                iy = max(y+1,1):min(y+h,h);
                B0 = B{refIndex,level}(iy,ix);  N0 = N{refIndex,level}(iy,ix);
                ix = max(1-x,1):min(w-x,w);
                iy = max(1-y,1):min(h-y,h);
                B1 = B{i,level}(iy,ix);  N1 = N{i,level}(iy,ix);
                % Sum different bits and normalize
                n = length(ix)*length(iy);
                err = sum(sum((xor(B0,B1) & N0 & N1))) / n;
                if err < minerr
                    minerr = err;
                    minx = x;  miny = y;
                end
            end
        end
        px = minx;  py = miny;
    end
    offsets{i} = [px py];
end

% Create aligned images
left = 1; top = 1;
[h,w,c] = size(images{refIndex});
bottom = h; right = w;
for i = others
    p = offsets{i};
    left = max(p(1)+1,left);
    right = min(w+p(1),right);
    top = max(p(2)+1,top);
    bottom = min(h+p(2),bottom);
end
aligned = cell(P,1);
for i = 1:P
    p = offsets{i};
    ix = (left-p(1)):min(right-p(1),w);
    iy = (top-p(2)):min(bottom-p(2),h);
    aligned{i} = images{i}(iy,ix,:);
end
